# beetbase shiny app
if(!require(shiny)) install.packages("shiny", repos = "http://cran.us.r-project.org")
if(!require(shinydashboard)) install.packages("shinydashboard", repos = "http://cran.us.r-project.org")
if(!require(bslib)) install.packages("bslib", repos = "http://cran.us.r-project.org")
if(!require(shinyjs)) install.packages("shinyjs", repos = "http://cran.us.r-project.org")
if(!require(rintrojs)) install.packages("rintrojs", repos = "http://cran.us.r-project.org")
if(!require(ggplot2)) install.packages("ggplot2", repos = "http://cran.us.r-project.org")
if(!require(ggiraph)) install.packages("ggiraph", repos = "http://cran.us.r-project.org")
if(!require(tidyr)) install.packages("tidyr", repos = "http://cran.us.r-project.org")
if(!require(dplyr)) install.packages("dplyr", repos = "http://cran.us.r-project.org")
if(!require(cowplot)) install.packages("cowplot", repos = "http://cran.us.r-project.org")
if(!require(htmlwidgets)) install.packages("htmlwidgets", repos = "http://cran.us.r-project.org")
if(!require(leaflet)) install.packages("leaflet", repos = "http://cran.us.r-project.org")
if(!require(grid)) install.packages("grid", repos = "http://cran.us.r-project.org")
if(!require(DT)) install.packages("DT", repos = "http://cran.us.r-project.org")
if(!require(JBrowseR)) install.packages("JBrowseR", repos = "http://cran.us.r-project.org")

########## set color for each species ##########
species_color = c('Beta_vulgaris_L_subsp_vulgaris'='#800000', 
  'Beta_vulgaris_L_subsp_vulgaris'='#800000', 
  'Beta_vulgaris_L'='#800000',
  'Beta_vulgaris_L_subsp_maritima'='#9A6324',
  'Beta_macrocarpa'="#808000",
  'Beta_trigyna'='#469990',
  'Beta_nana'='#000075',
  'Beta_hybr'='#e6194B',
  'Beta_patula'='#f58231',
  'Beta_lomatogona'='#ffe119',
  'Beta_macrorhiza'='#bfef45',
  'Beta_corolliflora'='#3cb44b',
  'Patellifolia_patellaris'='#42d4f4',
  'Patellifolia_procumbens'='#4363d8',
  'Beta_intermedia'='#911eb4',
  'Patellifolia_webbiana'='#f032e6')

############## GRIN dataset ###############

GRIN_data <- read.csv(file = 'data/DescriptorsGRIN_Global.csv', sep = ",", header = TRUE)
drops <- c("X", "AVAILABILITY", "ID")
GRIN_data_cleaned <- dplyr::select(GRIN_data, -c("X", "AVAILABILITY", "ID"))
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(CERCOSPORA, c('CERCOSPORA', 'CERCOSPORA_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(CURLY.TOP, c('CURLY_TOP', 'CURLY_TOP_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(RHIZOCTONIA, c('RHIZOCTONIA', 'RHIZOCTONIA_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(RHIZOMANIA, c('RHIZOMANIA', 'RHIZOMANIA_scale'), sep = ":")


############## RCRR DI over time ###############
DI_over_time <- read.csv(file = 'data/DI_over_time.csv', sep = ",", header = TRUE)
colnames(DI_over_time) = c("Year", "Experiment", "Type", "Company", "Source", "Rep", 
                           "plants", "0's", "1's", "2's", "3's", "4's", "5's", "6's", 
                           "7's", "DI", "Healthy", "Harvestable")
DI_over_time_short <- select(DI_over_time, -Healthy, -Harvestable)
DI_over_time_short$DI <- as.numeric(DI_over_time_short$DI) 


########## Genome data ############
EL10_chr_names <- c("EL10_2_Chr1","EL10_2_Chr2","EL10_2_Chr3","EL10_2_Chr3_s1","EL10_2_Chr4",
                    "EL10_2_Chr5","EL10_2_Chr5_s1","EL10_2_Chr5_s2","EL10_2_Chr6","EL10_2_Chr7",
                    "EL10_2_Chr8","EL10_2_Chr9","EL10_2_Chr_UnSca1","EL10_2_Chr_UnSca2",
                    "EL10_2_Chr_UnSca3","EL10_2_Chr_UnSca4","EL10_2_Chr_UnSca5","EL10_2_Chr_UnSca6")


########## Define Shiny UI for application that draws a histogram ###########

ui <- dashboardPage(
  skin = "green",
  #title = "BeetBase",
  header = dashboardHeader(titleWidth = '100%',
			   disable = TRUE,
                           title = "BeetBase: A Sugar Beet Toolbox for Genetics, Genomics and Germplasm Research"
                           ),
  
  dashboardSidebar(sidebarMenu(id = "menu", sidebarMenuOutput("menu")), 
                   width = 200, collapsed = FALSE, disable = TRUE
                   #tags$script(JS("document.getElementsByClassName('sidebar-toggle')[0].style.visibility = 'hidden';"))
                   ), 
  
  ############################
  dashboardBody(
    tags$style('.content-wrapper  {
                 background-color: #FFFFFF;
      }'),
    fluidPage(
              h2("Rhizoctonia Crown and Root Rot resistance test data"),
              
              fluidRow(
                mainPanel(
                  column(4, selectInput("seed_code", label = h3("Seed code"), 
                                        choices = sort(unique(DI_over_time$Source)),
                                        multiple = FALSE)
                         ),
                  hr(),
                  column(6, girafeOutput("bar_plots", width = "100%",
                                               height = "150%"))
                  )
                ),
              mainPanel(
                hr(),
                fluidRow(
                  column(12, DT::dataTableOutput("germplasm_search_table"))
                  )       
      )
    )
  )
)


# -----------------------------------------------------------------------
# -------------------------------SERVER ---------------------------------
# -----------------------------------------------------------------------

# Define server logic required to draw a histogram
server <- function(input, output, session) {
  
  
  ############# make DI table ##############
  data_input <- reactive({
    if (input$seed_code!="") {
      seed_source = input$seed_code
      data = DI_over_time_short %>% 
        filter(Source %in% seed_source)
      data
    }
    else if (input$seed_code==""){
      seed_source = as.character(DI_over_time_short$Source[0])
      data = DI_over_time_short %>% 
        filter(Source %in% seed_source)
      data
    }
    
  })
  
  output$germplasm_search_table <- DT::renderDataTable(data_input(),
                                                      caption = htmltools::tags$caption(
                                                        style = 'caption-side: bottom; text-align: Left;',
                                                        htmltools::withTags(
                                                          div(HTML('*Data generated at USDA-ARS Sugarbeet and Soil Management Research Lab.')))),
                                                      options = list(scrollX = TRUE),
                                                      rownames = FALSE)
  

  ############# make plots ##############

  output$bar_plots <- renderGirafe({
    data = data_input()
    #pivot
    year_data_pivot <- pivot_longer(data, cols = c("0's", "1's", "2's", "3's",
                                                   "4's", "5's", "6's", "7's"), 
                                    names_to = "Score_category")
    
  ########################### DI over time plot ####################
  year_data_pivot$Score_category <- as.factor(year_data_pivot$Score_category)
  year_data_pivot$Rep <- as.factor(year_data_pivot$Rep)
  year_data_pivot$DI <- as.numeric(year_data_pivot$DI)

  boxplot <- ggplot(data = year_data_pivot, aes(x = Score_category, y = value)) + 
    geom_boxplot(fill= 'lightgreen') +
    scale_x_discrete(name ="Disease Score Scale (0: Highly Resistant, 7: Highly Susceptible)", 
                     limits=c("0's", "1's", "2's", "3's", "4's", "5's", "6's", "7's")) +
    ggplot2::theme(panel.background = element_rect(fill = "grey92", colour = NA),
          plot.background = element_rect(fill = "transparent", colour = NA),
          plot.title = element_text(color="Black", size=14, face="bold.italic"),
          panel.grid = element_blank(),
          panel.border = element_blank(),
          legend.position="none") +
          ggtitle(paste("Accession/PI: ",as.character(input$seed_code))) + 
    labs(y = "Plants per disease rating")
  
  ggiraph::girafe(ggobj = boxplot)
  })
}

# Run the application 
shinyApp(ui, server, options = list(
  port = 7229)
  )
